@@ -22,13 +22,15 @@ gc.roundrectangle(140,20, 180,90, 5,5)
 gc.opacity(1)
 gc.roundrectangle(200,20, 240,90, 5,5)
 
+gc.font_weight(Magick::NormalWeight)
+gc.font_style(Magick::NormalStyle)
 gc.stroke('transparent')
 gc.fill('black')
 gc.gravity(Magick::SouthGravity)
-gc.text(-90,20, '"25%%"')
-gc.text(-30,20, '"50%%"')
-gc.text( 30,20, '"75%%"')
-gc.text( 90,20, '"100%%"')
+gc.text(-90,15, '"25%"')
+gc.text(-30,15, '"50%"')
+gc.text( 30,15, '"75%"')
+gc.text( 90,15, '"100%"')
 
 gc.draw(canvas)
 
