@@ -1,4 +1,4 @@
-/* $Id: rmagick.h,v 1.223 2008/01/28 22:31:50 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.268 2009/01/12 23:08:35 rmagick Exp $ */
 /*=============================================================================
 |               Copyright (C) 2008 by Timothy P. Hunter
 | Name:     rmagick.h
@@ -25,7 +25,11 @@
 #include <sys/types.h>
 #endif
 #include "ruby.h"
+#if defined(HAVE_RUBY_IO_H)
+#include "ruby/io.h"        // >= 1.9.0-5
+#else
 #include "rubyio.h"
+#endif
 
 
 // Undef Ruby's versions of these symbols
@@ -105,7 +109,7 @@
 #define OpenFile rb_io_t
 #endif
 
-// These macros are needed in 1.9.0 but aren't defined prior to 1.8.6.
+// These macros are required in 1.9.1 but aren't defined prior to 1.8.6.
 #if !defined(RSTRING_LEN)
 #define RSTRING_LEN(s) (RSTRING((s))->len)
 #endif
@@ -117,10 +121,37 @@
 #if !defined(RARRAY_LEN)
 #define RARRAY_LEN(a) RARRAY((a))->len
 #endif
+// Matz says this macro is read-only! (see http://www.ruby-forum.com/topic/146072)
 #if !defined(RARRAY_PTR)
 #define RARRAY_PTR(a) RARRAY((a))->ptr
 #endif
 
+
+// Convert a C string to a Ruby symbol. Used in marshal_dump/marshal_load methods
+#define CSTR2SYM(s) ID2SYM(rb_intern(s))
+// Convert a C string to a Ruby String, or nil if the ptr is NULL
+#define MAGICK_STRING_TO_OBJ(f) (f) ? rb_str_new2(f) : Qnil
+// Copy the C string in a Ruby String object to ImageMagick memory, or set
+// the pointer to NULL if the object is nil.
+#define OBJ_TO_MAGICK_STRING(f, obj) \
+    if ((obj) != Qnil)\
+        magick_clone_string(&f, RSTRING_PTR(obj));\
+    else\
+        f = NULL;
+
+
+// These two functions are defined in IM starting with 6.3.2.
+#if !defined(HAVE_GETSTRINGINFODATUM)
+#define GetStringInfoDatum(p) (unsigned char *)((p)->datum)
+#define GetStringInfoLength(p) (size_t)((p)->length)
+#endif
+
+// IM 6.4.1 replaced AllocateImage with AcquireImage.
+// Both have the same signature.
+#if !defined(HAVE_ACQUIREIMAGE)
+#define AcquireImage(info) AllocateImage(info)
+#endif
+
 // ImageLayerMethod replaced MagickLayerMethod starting with 6.3.6
 #if defined(HAVE_TYPE_IMAGELAYERMETHOD)
 #define LAYERMETHODTYPE ImageLayerMethod
@@ -226,7 +257,49 @@ typedef enum _QuantumExpressionOperator
     OrQuantumOperator,
     RShiftQuantumOperator,
     SubtractQuantumOperator,
-    XorQuantumOperator
+    XorQuantumOperator,
+#if defined(HAVE_ENUM_POWEVALUATEOPERATOR)
+    PowQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_LOGEVALUATEOPERATOR)
+    LogQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_THRESHOLDEVALUATEOPERATOR)
+    ThresholdQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_THRESHOLDBLACKEVALUATEOPERATOR)
+    ThresholdBlackQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_THRESHOLDWHITEEVALUATEOPERATOR)
+    ThresholdWhiteQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_GAUSSIANNOISEEVALUATEOPERATOR)
+    GaussianNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_IMPULSENOISEEVALUATEOPERATOR)
+    ImpulseNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_LAPLACIANNOISEEVALUATEOPERATOR)
+    LaplacianNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_MULTIPLICATIVENOISEEVALUATEOPERATOR)
+    MultiplicativeNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_POISSONNOISEEVALUATEOPERATOR)
+    PoissonNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_UNIFORMNOISEEVALUATEOPERATOR)
+    UniformNoiseQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_COSINEEVALUATEOPERATOR)
+    CosineQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_SINEEVALUATEOPERATOR)
+    SineQuantumOperator,
+#endif
+#if defined(HAVE_ENUM_ADDMODULUSEVALUATEOPERATOR)
+    AddModulusQuantumOperator
+#endif
 } QuantumExpressionOperator ;
 
 
@@ -252,6 +325,7 @@ EXTERN VALUE Class_DrawOptions;
 EXTERN VALUE Class_Image;
 EXTERN VALUE Class_Montage;
 EXTERN VALUE Class_ImageMagickError;
+EXTERN VALUE Class_FatalImageMagickError;
 EXTERN VALUE Class_DestroyedImageError;
 EXTERN VALUE Class_GradientFill;
 EXTERN VALUE Class_TextureFill;
@@ -287,6 +361,9 @@ EXTERN VALUE Class_DisposeType;
 #if defined(HAVE_DISTORTIMAGE)
 EXTERN VALUE Class_DistortImageMethod;
 #endif
+#if defined(HAVE_TYPE_DITHERMETHOD)
+EXTERN VALUE Class_DitherMethod;
+#endif
 EXTERN VALUE Class_EndianType;
 EXTERN VALUE Class_FilterTypes;
 EXTERN VALUE Class_GravityType;
@@ -300,6 +377,10 @@ EXTERN VALUE Class_PaintMethod;
 EXTERN VALUE Class_PreviewType;
 EXTERN VALUE Class_RenderingIntent;
 EXTERN VALUE Class_ResolutionType;
+#if defined(HAVE_SPARSECOLORIMAGE)
+EXTERN VALUE Class_SparseColorMethod;
+#endif
+EXTERN VALUE Class_SpreadMethod;
 EXTERN VALUE Class_StorageType;
 EXTERN VALUE Class_StretchType;
 EXTERN VALUE Class_StyleType;
@@ -310,17 +391,16 @@ EXTERN VALUE Class_VirtualPixelMethod;
 *   Commonly-used IDs
 */
 EXTERN ID rm_ID_trace_proc;        // "@trace_proc"
-EXTERN ID rm_ID__dummy_img_;       // "_dummy_img_"
 EXTERN ID rm_ID_call;              // "call"
 EXTERN ID rm_ID_changed;           // "changed"
 EXTERN ID rm_ID_cur_image;         // "cur_image"
 EXTERN ID rm_ID_dup;               // "dup"
-EXTERN ID rm_ID_enumerators;       // "enumerators"
 EXTERN ID rm_ID_fill;              // "fill"
 EXTERN ID rm_ID_flag;              // "flag"
 EXTERN ID rm_ID_from_s;            // "from_s"
 EXTERN ID rm_ID_Geometry;          // "Geometry"
 EXTERN ID rm_ID_GeometryValue;     // "GeometryValue"
+EXTERN ID rm_ID_has_key_q;         // "has_key?"
 EXTERN ID rm_ID_height;            // "height"
 EXTERN ID rm_ID_initialize_copy;   // "initialize_copy"
 EXTERN ID rm_ID_length;            // "length"
@@ -328,7 +408,6 @@ EXTERN ID rm_ID_notify_observers;  // "notify_observers"
 EXTERN ID rm_ID_new;               // "new"
 EXTERN ID rm_ID_push;              // "push"
 EXTERN ID rm_ID_spaceship;         // "<=>
-EXTERN ID rm_ID__tmpnam_;          // "_tmpnam_"
 EXTERN ID rm_ID_to_i;              // "to_i"
 EXTERN ID rm_ID_to_s;              // "to_s"
 EXTERN ID rm_ID_values;            // "values"
@@ -499,7 +578,7 @@ Pixel_##_cmyk_channel_(VALUE self) \
    _cls =  Class_##tag = rm_define_enum_type(#tag);
 
 #define ENUMERATOR(val)\
-   _enum = rm_enum_new(_cls, ID2SYM(rb_intern(#val)), INT2FIX(val));\
+   _enum = rm_enum_new(_cls, ID2SYM(rb_intern(#val)), INT2NUM(val));\
    rb_define_const(Module_Magick, #val, _enum);
 #define END_ENUM }
 
@@ -532,6 +611,20 @@ Pixel_##_cmyk_channel_(VALUE self) \
 // declaration can refer to a function defined in another source file or
 // the same source file.
 
+
+// rmmain.c
+extern void Init_RMagick2(void);
+
+
+// rmagick.c
+extern VALUE Magick_colors(VALUE);
+extern VALUE Magick_fonts(VALUE);
+extern VALUE Magick_init_formats(VALUE);
+extern VALUE Magick_limit_resource(int, VALUE *, VALUE);
+extern VALUE Magick_set_cache_threshold(VALUE, VALUE);
+extern VALUE Magick_set_log_event_mask(int, VALUE *, VALUE);
+extern VALUE Magick_set_log_format(VALUE, VALUE);
+
 // rmdraw.c
 ATTR_WRITER(Draw, affine)
 ATTR_WRITER(Draw, align)
@@ -547,6 +640,8 @@ ATTR_WRITER(Draw, font_stretch)
 ATTR_WRITER(Draw, font_style)
 ATTR_WRITER(Draw, font_weight)
 ATTR_WRITER(Draw, gravity)
+ATTR_WRITER(Draw, interword_spacing)
+ATTR_WRITER(Draw, kerning)
 ATTR_WRITER(Draw, pointsize)
 ATTR_WRITER(Draw, rotation)
 ATTR_WRITER(Draw, stroke)
@@ -555,6 +650,7 @@ ATTR_WRITER(Draw, stroke_width)
 ATTR_WRITER(Draw, text_antialias)
 ATTR_WRITER(Draw, tile)
 ATTR_WRITER(Draw, undercolor)
+extern VALUE Draw_alloc(VALUE);
 extern VALUE Draw_annotate(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Draw_clone(VALUE);
 extern VALUE Draw_composite(int, VALUE *, VALUE);
@@ -565,12 +661,21 @@ extern VALUE Draw_get_type_metrics(int, VALUE *, VALUE);
 extern VALUE Draw_init_copy(VALUE, VALUE);
 extern VALUE Draw_initialize(VALUE);
 extern VALUE Draw_inspect(VALUE);
-extern VALUE Draw_alloc(VALUE);
-extern VALUE DrawOptions_alloc(VALUE);
+extern VALUE Draw_marshal_dump(VALUE);
+extern VALUE Draw_marshal_load(VALUE, VALUE);
 extern VALUE Draw_primitive(VALUE, VALUE);
+extern VALUE DrawOptions_alloc(VALUE);
 extern VALUE DrawOptions_initialize(VALUE);
 
 
+extern VALUE PolaroidOptions_alloc(VALUE);
+extern VALUE PolaroidOptions_initialize(VALUE);
+extern VALUE rm_polaroid_new(void);
+ATTR_WRITER(PolaroidOptions, shadow_color);
+ATTR_WRITER(PolaroidOptions, border_color);
+
+
+// rmmontage.c
 ATTR_WRITER(Montage, background_color)
 ATTR_WRITER(Montage, border_color)
 ATTR_WRITER(Montage, border_width)
@@ -593,18 +698,7 @@ extern VALUE Montage_alloc(VALUE);
 extern VALUE rm_montage_new(void);
 
 
-extern VALUE PolaroidOptions_alloc(VALUE);
-extern VALUE PolaroidOptions_initialize(VALUE);
-extern VALUE rm_polaroid_new(void);
-ATTR_WRITER(PolaroidOptions, shadow_color);
-ATTR_WRITER(PolaroidOptions, border_color);
-
-
-// rmmain.c
-extern VALUE rm_montage_new(void);
-
-
- // rmilist.c
+// rmilist.c
 extern VALUE ImageList_animate(int, VALUE *, VALUE);
 extern VALUE ImageList_append(VALUE, VALUE);
 extern VALUE ImageList_average(VALUE);
@@ -620,6 +714,7 @@ extern VALUE ImageList_morph(VALUE, VALUE);
 extern VALUE ImageList_mosaic(VALUE);
 extern VALUE ImageList_optimize_layers(VALUE, VALUE);
 extern VALUE ImageList_quantize(int, VALUE*, VALUE);
+extern VALUE ImageList_remap(int, VALUE *, VALUE);
 extern VALUE ImageList_to_blob(VALUE);
 extern VALUE ImageList_write(VALUE, VALUE);
 
@@ -641,6 +736,8 @@ ATTR_ACCESSOR(Info, density)
 ATTR_ACCESSOR(Info, depth)
 ATTR_ACCESSOR(Info, dispose)
 ATTR_ACCESSOR(Info, dither)
+ATTR_ACCESSOR(Info, encoding)
+ATTR_ACCESSOR(Info, endian)
 ATTR_ACCESSOR(Info, extract)
 ATTR_ACCESSOR(Info, filename)
 ATTR_ACCESSOR(Info, fill)
@@ -671,6 +768,7 @@ ATTR_ACCESSOR(Info, stroke)
 ATTR_ACCESSOR(Info, stroke_width)
 ATTR_WRITER(Info, texture)
 ATTR_ACCESSOR(Info, tile_offset)
+ATTR_ACCESSOR(Info, transparent_color)
 ATTR_ACCESSOR(Info, undercolor)
 ATTR_ACCESSOR(Info, units)
 ATTR_ACCESSOR(Info, view)
@@ -685,7 +783,8 @@ extern VALUE Info_channel(int, VALUE *, VALUE);
 extern VALUE Info_undefine(VALUE, VALUE, VALUE);
 extern VALUE Info_initialize(VALUE);
 extern VALUE rm_info_new(void);
-
+extern DisposeType rm_dispose_to_enum(const char *);
+extern GravityType rm_gravity_to_enum(const char *);
 
 // rmimage.c
 ATTR_WRITER(Image, alpha)
@@ -721,11 +820,11 @@ ATTR_ACCESSOR(Image, fuzz)
 ATTR_ACCESSOR(Image, gamma)
 ATTR_ACCESSOR(Image, geometry)
 ATTR_ACCESSOR(Image, gravity)
-ATTR_READER(Image, image_type)
+ATTR_ACCESSOR(Image, image_type)
 ATTR_ACCESSOR(Image, interlace)
 ATTR_ACCESSOR(Image, iptc_profile)
 ATTR_ACCESSOR(Image, iterations)
-ATTR_ACCESSOR(Image, mask)
+ATTR_WRITER(Image, mask)
 ATTR_ACCESSOR(Image, matte)
 ATTR_ACCESSOR(Image, matte_color)
 ATTR_READER(Image, mean_error_per_pixel)
@@ -748,6 +847,7 @@ ATTR_READER(Image, scene)
 ATTR_ACCESSOR(Image, start_loop)
 ATTR_ACCESSOR(Image, ticks_per_second)
 ATTR_READER(Image, total_colors)
+ATTR_READER(Image, total_ink_density)
 ATTR_ACCESSOR(Image, transparent_color)
 ATTR_ACCESSOR(Image, units)
 ATTR_ACCESSOR(Image, virtual_pixel_method)
@@ -756,6 +856,7 @@ ATTR_ACCESSOR(Image, y_resolution)
 
 extern ChannelType extract_channels(int *, VALUE *);
 extern void raise_ChannelType_error(VALUE);
+extern void add_format_prefix(Info *, VALUE);
 
 extern VALUE Image_alloc(VALUE);
 extern VALUE Image_initialize(int, VALUE *, VALUE);
@@ -765,10 +866,13 @@ extern VALUE Image_adaptive_resize(int, VALUE *, VALUE);
 extern VALUE Image_adaptive_sharpen(int, VALUE *, VALUE);
 extern VALUE Image_adaptive_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_adaptive_threshold(int, VALUE *, VALUE);
+extern VALUE Image_add_compose_mask(VALUE, VALUE);
 extern VALUE Image_add_noise(VALUE, VALUE);
 extern VALUE Image_add_noise_channel(int, VALUE *, VALUE);
 extern VALUE Image_add_profile(VALUE, VALUE);
 extern VALUE Image_affine_transform(VALUE, VALUE);
+extern VALUE Image_alpha(int, VALUE *, VALUE);
+extern VALUE Image_alpha_q(VALUE);
 extern VALUE Image_aref(VALUE, VALUE);
 extern VALUE Image_aset(VALUE, VALUE, VALUE);
 extern VALUE Image_auto_orient(VALUE);
@@ -800,10 +904,12 @@ extern VALUE Image_colorize(int, VALUE *, VALUE);
 extern VALUE Image_colormap(int, VALUE *, VALUE);
 extern VALUE Image_combine(int, VALUE *, VALUE);
 extern VALUE Image_composite(int, VALUE *, VALUE);
+extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
 extern VALUE Image_composite_bang(int, VALUE *, VALUE);
 extern VALUE Image_composite_channel(int, VALUE *, VALUE);
 extern VALUE Image_composite_channel_bang(int, VALUE *, VALUE);
-extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
+extern VALUE Image_composite_tiled(int, VALUE *, VALUE);
+extern VALUE Image_composite_tiled_bang(int, VALUE *, VALUE);
 extern VALUE Image_compress_colormap_bang(VALUE);
 extern VALUE Image_constitute(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_contrast(int, VALUE *, VALUE);
@@ -814,7 +920,11 @@ extern VALUE Image_copy(VALUE);
 extern VALUE Image_crop(int, VALUE *, VALUE);
 extern VALUE Image_crop_bang(int, VALUE *, VALUE);
 extern VALUE Image_cycle_colormap(VALUE, VALUE);
+extern VALUE Image_decipher(VALUE, VALUE);
+extern VALUE Image_define(VALUE, VALUE, VALUE);
 extern VALUE Image_delete_profile(VALUE, VALUE);
+extern VALUE Image_delete_compose_mask(VALUE);
+extern VALUE Image_deskew(int, VALUE *, VALUE);
 extern VALUE Image_despeckle(VALUE);
 extern VALUE Image_destroy_bang(VALUE);
 extern VALUE Image_destroyed_q(VALUE);
@@ -830,6 +940,7 @@ extern VALUE Image_dup(VALUE);
 extern VALUE Image_each_profile(VALUE);
 extern VALUE Image_edge(int, VALUE *, VALUE);
 extern VALUE Image_emboss(int, VALUE *, VALUE);
+extern VALUE Image_encipher(VALUE, VALUE);
 extern VALUE Image_enhance(VALUE);
 extern VALUE Image_equalize(VALUE);
 extern VALUE Image_equalize_channel(int, VALUE *, VALUE);
@@ -859,12 +970,17 @@ extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
 extern VALUE Image_level2(int, VALUE *, VALUE);
 extern VALUE Image_level_channel(int, VALUE *, VALUE);
+extern VALUE Image_level_colors(int, VALUE *, VALUE);
+extern VALUE Image_levelize_channel(int, VALUE *, VALUE);
 extern VALUE Image_linear_stretch(int, VALUE *, VALUE);
 extern VALUE Image_liquid_rescale(int, VALUE *, VALUE);
 extern VALUE Image__load(VALUE, VALUE);
 extern VALUE Image_magnify(VALUE);
 extern VALUE Image_magnify_bang(VALUE);
 extern VALUE Image_map(int, VALUE *, VALUE);
+extern VALUE Image_marshal_dump(VALUE);
+extern VALUE Image_marshal_load(VALUE, VALUE);
+extern VALUE Image_mask(int, VALUE *, VALUE);
 extern VALUE Image_matte_flood_fill(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_median_filter(int, VALUE *, VALUE);
 extern VALUE Image_minify(VALUE);
@@ -900,6 +1016,7 @@ extern VALUE Image_read(VALUE, VALUE);
 extern VALUE Image_read_inline(VALUE, VALUE);
 extern VALUE Image_recolor(VALUE, VALUE);
 extern VALUE Image_reduce_noise(VALUE, VALUE);
+extern VALUE Image_remap(int, VALUE *, VALUE);
 extern VALUE Image_resize(int, VALUE *, VALUE);
 extern VALUE Image_resize_bang(int, VALUE *, VALUE);
 extern VALUE Image_roll(VALUE, VALUE, VALUE);
@@ -925,6 +1042,7 @@ extern VALUE Image_signature(VALUE);
 extern VALUE Image_sketch(int, VALUE *, VALUE);
 extern VALUE Image_solarize(int, VALUE *, VALUE);
 extern VALUE Image_spaceship(VALUE, VALUE);
+extern VALUE Image_sparse_color(int, VALUE *, VALUE);
 extern VALUE Image_splice(int, VALUE *, VALUE);
 extern VALUE Image_spread(int, VALUE *, VALUE);
 extern VALUE Image_stegano(VALUE, VALUE, VALUE);
@@ -941,12 +1059,14 @@ extern VALUE Image_tint(int, VALUE *, VALUE);
 extern VALUE Image_to_blob(VALUE);
 extern VALUE Image_to_color(VALUE, VALUE);
 extern VALUE Image_transparent(int, VALUE *, VALUE);
+extern VALUE Image_transparent_chroma(int, VALUE *, VALUE);
 extern VALUE Image_transpose(VALUE);
 extern VALUE Image_transpose_bang(VALUE);
 extern VALUE Image_transverse(VALUE);
 extern VALUE Image_transverse_bang(VALUE);
 extern VALUE Image_trim(int, VALUE *, VALUE);
 extern VALUE Image_trim_bang(int, VALUE *, VALUE);
+extern VALUE Image_undefine(VALUE, VALUE);
 extern VALUE Image_unique_colors(VALUE);
 extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
 extern VALUE Image_unsharp_mask_channel(int, VALUE *, VALUE);
@@ -972,28 +1092,8 @@ extern VALUE  TextureFill_initialize(VALUE, VALUE);
 extern VALUE  TextureFill_fill(VALUE, VALUE);
 
 
-// rmutil.c
-extern VALUE  ChromaticityInfo_to_s(VALUE);
-extern VALUE  ChromaticityInfo_new(ChromaticityInfo *);
-extern void   Color_to_PixelPacket(PixelPacket *, VALUE);
-extern void   Color_to_MagickPixelPacket(Image *, MagickPixelPacket *, VALUE);
-extern VALUE  Color_to_s(VALUE);
-extern VALUE  Color_from_ColorInfo(const ColorInfo *);
-extern VALUE  ClassType_new(ClassType);
-extern VALUE  ColorspaceType_new(ColorspaceType);
-extern VALUE  CompositeOperator_new(CompositeOperator);
-extern VALUE  CompressionType_new(CompressionType);
-extern VALUE  DisposeType_new(DisposeType);
-extern VALUE  EndianType_new(EndianType);
-extern VALUE  FilterTypes_new(FilterTypes);
-extern VALUE  GravityType_new(GravityType);
-extern VALUE  Font_to_s(VALUE);
-extern VALUE  ImageList_cur_image(VALUE);
-extern VALUE  ImageMagickError_initialize(int, VALUE *, VALUE);
-extern VALUE  ImageType_new(ImageType);
-extern VALUE  InterlaceType_new(InterlaceType);
+// rmpixel.c
 
-extern VALUE   Pixel_alloc(VALUE);
 
 ATTR_ACCESSOR(Pixel, red)
 ATTR_ACCESSOR(Pixel, green)
@@ -1003,6 +1103,8 @@ ATTR_ACCESSOR(Pixel, cyan)
 ATTR_ACCESSOR(Pixel, magenta)
 ATTR_ACCESSOR(Pixel, yellow)
 ATTR_ACCESSOR(Pixel, black)
+extern void   destroy_Pixel(Pixel *);
+extern VALUE  Pixel_alloc(VALUE);
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
 extern VALUE  Pixel_dup(VALUE);
@@ -1010,44 +1112,21 @@ extern VALUE  Pixel_eql_q(VALUE, VALUE);
 extern VALUE  Pixel_fcmp(int, VALUE *, VALUE);
 extern VALUE  Pixel_from_color(VALUE, VALUE);
 extern VALUE  Pixel_from_HSL(VALUE, VALUE);
+extern VALUE  Pixel_from_hsla(int, VALUE *, VALUE);
 extern VALUE  Pixel_hash(VALUE);
 extern VALUE  Pixel_initialize(int, VALUE *, VALUE);
 extern VALUE  Pixel_init_copy(VALUE, VALUE);
 extern VALUE  Pixel_intensity(VALUE);
+extern VALUE  Pixel_marshal_dump(VALUE);
+extern VALUE  Pixel_marshal_load(VALUE, VALUE);
 extern VALUE  Pixel_spaceship(VALUE, VALUE);
 extern VALUE  Pixel_to_color(int, VALUE *, VALUE);
 extern VALUE  Pixel_to_HSL(VALUE);
+extern VALUE  Pixel_to_hsla(VALUE);
 extern VALUE  Pixel_to_s(VALUE);
-extern VALUE  PixelPacket_to_Color_Name(Image *, PixelPacket *);
-extern VALUE  PixelPacket_to_Color_Name_Info(Info *, PixelPacket *);
-extern VALUE  Pixel_from_PixelPacket(PixelPacket *);
 
-extern void   Point_to_PointInfo(PointInfo *, VALUE);
-extern VALUE  PointInfo_to_Point(PointInfo *);
-extern VALUE  PrimaryInfo_to_s(VALUE);
-extern VALUE  PrimaryInfo_from_PrimaryInfo(PrimaryInfo *);
-extern VALUE  RectangleInfo_to_s(VALUE);
-extern VALUE  Rectangle_from_RectangleInfo(RectangleInfo *);
-extern VALUE  RenderingIntent_new(RenderingIntent);
-extern VALUE  ResolutionType_new(ResolutionType);
-extern VALUE  SegmentInfo_to_s(VALUE);
-extern VALUE  Segment_from_SegmentInfo(SegmentInfo *);
-extern void   AffineMatrix_to_AffineMatrix(AffineMatrix *, VALUE);
-extern void   ChromaticityInfo_to_ChromaticityInfo(ChromaticityInfo *, VALUE);
-extern void   Color_to_ColorInfo(ColorInfo *, VALUE);
-extern VALUE  InterpolatePixelMethod_new(InterpolatePixelMethod);
-extern VALUE  OrientationType_new(OrientationType);
-extern void   PrimaryInfo_to_PrimaryInfo(PrimaryInfo *, VALUE);
-extern void   Rectangle_to_RectangleInfo(RectangleInfo *, VALUE);
-extern void   Segment_to_SegmentInfo(SegmentInfo *, VALUE);
-extern void   Font_to_TypeInfo(TypeInfo *, VALUE);
-extern void   TypeMetric_to_TypeMetric(TypeMetric *, VALUE);
-extern VALUE  Font_from_TypeInfo(TypeInfo *);
-extern VALUE  TypeMetric_to_s(VALUE);
-extern VALUE  TypeMetric_from_TypeMetric(TypeMetric *);
-extern const char *StorageType_name(StorageType);
-extern VALUE  VirtualPixelMethod_new(VirtualPixelMethod);
 
+// rmenum.c
 extern VALUE  Enum_alloc(VALUE);
 extern VALUE  Enum_initialize(VALUE, VALUE, VALUE);
 extern VALUE  Enum_to_s(VALUE);
@@ -1056,32 +1135,83 @@ extern VALUE  Enum_spaceship(VALUE, VALUE);
 extern VALUE  Enum_case_eq(VALUE, VALUE);
 extern VALUE  Enum_type_initialize(VALUE, VALUE, VALUE);
 extern VALUE  Enum_type_each(VALUE);
+extern VALUE  rm_enum_new(VALUE, VALUE, VALUE);
 
 
+// rmstruct.c
+extern VALUE  ChromaticityInfo_to_s(VALUE);
+extern VALUE  ChromaticityInfo_new(ChromaticityInfo *);
+extern void   Color_to_PixelPacket(PixelPacket *, VALUE);
+extern void   Color_to_MagickPixelPacket(Image *, MagickPixelPacket *, VALUE);
+extern VALUE  Color_to_s(VALUE);
+extern VALUE  Import_ColorInfo(const ColorInfo *);
+extern VALUE  ClassType_new(ClassType);
+extern VALUE  ColorspaceType_new(ColorspaceType);
+extern VALUE  CompositeOperator_new(CompositeOperator);
+extern VALUE  CompressionType_new(CompressionType);
+extern VALUE  DisposeType_new(DisposeType);
+extern VALUE  EndianType_new(EndianType);
+extern VALUE  FilterTypes_new(FilterTypes);
+extern VALUE  GravityType_new(GravityType);
+extern VALUE  Font_to_s(VALUE);
+extern VALUE  ImageType_new(ImageType);
+extern VALUE  InterlaceType_new(InterlaceType);
+extern VALUE  Pixel_from_MagickPixelPacket(const MagickPixelPacket *);
+extern VALUE  Pixel_from_PixelPacket(const PixelPacket *);
+extern void   Export_PointInfo(PointInfo *, VALUE);
+extern VALUE  Import_PointInfo(PointInfo *);
+extern VALUE  PrimaryInfo_to_s(VALUE);
+extern VALUE  Import_PrimaryInfo(PrimaryInfo *);
+extern VALUE  RectangleInfo_to_s(VALUE);
+extern VALUE  Import_RectangleInfo(RectangleInfo *);
+extern VALUE  RenderingIntent_new(RenderingIntent);
+extern VALUE  ResolutionType_new(ResolutionType);
+extern VALUE  SegmentInfo_to_s(VALUE);
+extern VALUE  Import_SegmentInfo(SegmentInfo *);
+extern void   Export_AffineMatrix(AffineMatrix *, VALUE);
+extern VALUE  Import_AffineMatrix(AffineMatrix *);
+extern void   Export_ChromaticityInfo(ChromaticityInfo *, VALUE);
+extern void   Export_ColorInfo(ColorInfo *, VALUE);
+extern VALUE  InterpolatePixelMethod_new(InterpolatePixelMethod);
+extern VALUE  OrientationType_new(OrientationType);
+extern void   Export_PrimaryInfo(PrimaryInfo *, VALUE);
+extern void   Export_RectangleInfo(RectangleInfo *, VALUE);
+extern void   Export_SegmentInfo(SegmentInfo *, VALUE);
+extern void   Font_to_TypeInfo(TypeInfo *, VALUE);
+extern void   Export_TypeMetric(TypeMetric *, VALUE);
+extern VALUE  Import_TypeInfo(const TypeInfo *);
+extern VALUE  TypeMetric_to_s(VALUE);
+extern VALUE  Import_TypeMetric(TypeMetric *);
+extern const char *StorageType_name(StorageType);
+extern VALUE  VirtualPixelMethod_new(VirtualPixelMethod);
+extern VALUE  LAYERMETHODTYPE_NEW(LAYERMETHODTYPE);
+
+
+// rmutil.c
+extern VALUE  ImageMagickError_initialize(int, VALUE *, VALUE);
 extern void  *magick_malloc(const size_t);
 extern void  *magick_safe_malloc(const size_t, const size_t);
 extern void   magick_free(void *);
 extern void  *magick_realloc(void *, const size_t);
 extern void  *magick_safe_realloc(void *, const size_t, const size_t);
 extern void   magick_clone_string(char **, const char *);
-extern VALUE  rm_enum_new(VALUE, VALUE, VALUE);
+extern VALUE  rm_cur_image(VALUE);
+extern VALUE  rm_pixelpacket_to_color_name(Image *, PixelPacket *);
+extern VALUE  rm_pixelpacket_to_color_name_info(Info *, PixelPacket *);
 extern VALUE  rm_no_freeze(VALUE);
-#if !defined(StringValuePtr)
-extern char  *rm_string_value_ptr(volatile VALUE *);
-#endif
-extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
 extern int    rm_strcasecmp(const char *, const char *);
 extern int    rm_strncasecmp(const char *, const char *, size_t);
 extern void   rm_check_ary_len(VALUE, long);
 extern Image *rm_check_destroyed(VALUE);
 extern Image *rm_check_frozen(VALUE);
+extern VALUE  rm_to_s(VALUE);
 extern char  *rm_str2cstr(VALUE, long *);
 extern int    rm_check_num2dbl(VALUE);
 extern double rm_fuzz_to_dbl(VALUE);
 extern Quantum rm_app2quantum(VALUE);
 extern double rm_percentage(VALUE);
 extern double rm_str_to_pct(VALUE);
-extern VALUE  rm_define_enum_type(char *);
+extern VALUE  rm_define_enum_type(const char *);
 extern void   rm_write_temp_image(Image *, char *);
 extern void   rm_delete_temp_image(char *);
 extern void   rm_not_implemented(void);
@@ -1089,6 +1219,9 @@ extern void   rm_attr_write(VALUE, VALUE);
 extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
 extern const char *rm_get_property(const Image *, const char *);
 extern MagickBooleanType rm_set_property(Image *, const char *, const char *);
+extern void   rm_set_user_artifact(Image *, Info *);
+void          rm_set_magick_pixel_packet(Pixel *, IndexPacket *, MagickPixelPacket *);
+extern void   rm_sync_image_options(Image *, Info *);
 extern void   rm_split(Image *);
 extern void   rm_magick_error(const char *, const char *);
 
@@ -1105,6 +1238,9 @@ extern Image *rm_clone_image(Image *);
 extern MagickBooleanType rm_progress_monitor(const char *, const MagickOffsetType, const MagickSizeType, void *);
 extern VALUE  rm_exif_by_entry(Image *);
 extern VALUE  rm_exif_by_number(Image *);
-
+extern void   rm_get_optional_arguments(VALUE);
+extern void   rm_fatal_error_handler(const ExceptionType, const char *, const char *);
+extern void   rm_error_handler(const ExceptionType, const char *, const char *);
+extern void   rm_warning_handler(const ExceptionType, const char *, const char *);
 #endif
 
